@@ -345,6 +345,12 @@ class TextResource(ResourceBase):
         return self.last_modified
 
 
+class NotInPack(Exception):
+    def __init__(self, name, pack):
+        self.name = name
+        self.pack = pack
+        super(NotInPack, self).__init__('{name}: not in pack {pack}'.format(name=name, pack=pack))
+
 class PackBase(object):
     """Base class for a texture pack.
 
@@ -404,6 +410,9 @@ class PackBase(object):
         return any(self.get_resource(n).is_modified_since(then)
                 for n in self.get_resource_names())
 
+    def __str__(self):
+        return self.__unicode__().encode('UTF-8')
+
 
 class RecipePack(PackBase):
     """A texture pack assembled from other resources."""
@@ -420,7 +429,10 @@ class RecipePack(PackBase):
         self.resources[resource.name] = resource
 
     def get_resource(self, name):
-        return self.resources[name]
+        try:
+            return self.resources[name]
+        except KeyError:
+            raise NotInPack(name, self)
 
     def get_resource_names(self):
         return self.resources.keys()
@@ -431,6 +443,9 @@ class RecipePack(PackBase):
                 return True
         return False
 
+    def __unicode__(self):
+        return '<RecipePack {0!r}>'.format(self.label)
+
 
 class SourcePack(PackBase):
     """A texture pack that gets resources from a ZIP file or directory."""
@@ -622,8 +637,11 @@ class MapBase(object):
         pass
 
 class NotInMap(Exception):
-    def __init__(self, name):
-        super(NotInMap, self).__init__('{0!r} not found in map'.format(name))
+    def __init__(self, name, map):
+        super(NotInMap, self).__init__('{0!r} not found in map (must be one of {1})'.format(
+            name, ', '.join(map.names)))
+        self.name = name
+        self.maop = map
 
 class GridMap(MapBase):
     def __init__(self, source_box, cell_box, names):
@@ -654,10 +672,11 @@ class GridMap(MapBase):
         try:
             i = self.names.index(name)
         except ValueError:
-            raise NotInMap(name)
+            raise NotInMap(name, self)
         u, v = i % self.nx, i // self.nx
-        return (self.im_left + self.cell_wd * u, self.im_top + self.cell_ht * v,
+        result = (self.im_left + self.cell_wd * u, self.im_top + self.cell_ht * v,
             self.im_left + self.cell_wd * (u + 1), self.im_top + self.cell_ht * (v + 1))
+        return result
 
 class CompositeMap(MapBase):
     """A map that combines several other maps.
@@ -673,7 +692,7 @@ class CompositeMap(MapBase):
                 return m.get_box(name)
             except NotInMap:
                 pass
-        raise NotInMap(name)
+        raise NotInMap(name, self)
 
     @property
     def names(self):
@@ -938,14 +957,25 @@ class Mixer(object):
             A new pack object (subclass of PackBase).
         """
 
+        # Check we have been supplied with the parameters we have declared.
+        # Also unjumble any parameters marked for this odd treatment.
+        # XXX this will affect uses of the same param in later calls. Bad?
         param_specss = recipe.get('parameters')
         if param_specss:
             pack_specs = param_specss.get('packs')
             if pack_specs:
                 for param_spec in pack_specs:
-                    if param_spec not in self.packs:
+                    if isinstance(param_spec, basestring):
+                        if param_spec not in self.packs:
+                            raise MissingParameter(param_spec)
+                        continue
+                    param_name = param_spec['name']
+                    if param_name not in self.packs:
                         raise MissingParameter(param_spec)
-
+                    if 'unjumble' in param_spec:
+                        unjumbled_atlas = self.get_atlas(param_spec['unjumble'], base=base)
+                        unjumbled_pack = self.make_unjumbled_pack(self.packs[param_name], unjumbled_atlas)
+                        self.packs[param_name] = unjumbled_pack
 
         if 'maps' in recipe:
             self.get_atlas(recipe['maps'], base, self.atlas)
@@ -983,7 +1013,6 @@ class Mixer(object):
     def iter_resources(self, src_pack, resources_spec, base):
         """Given a files spec, yield a sequence of resources.
 
-
         """
         for file_spec in resources_spec:
             if isinstance(file_spec, basestring):
@@ -1003,7 +1032,13 @@ class Mixer(object):
                     res_name = file_spec.get('file', 'pack.png')
                 else:
                     res_name = file_spec['file']
-                src_res = src_pack.get_resource(file_spec.get('source', res_name))
+                try:
+                    src_res = src_pack.get_resource(file_spec.get('source', res_name))
+                except NotInPack:
+                    if file_spec.get('if_missing') == 'relax':
+                        continue
+                    else:
+                        raise
                 if 'replace' in file_spec:
                     src_map = self.get_map(src_pack.atlas, file_spec.get('map', src_res.name), base)
                     res = CompositeResource(res_name, src_res, src_map)
